@@ -16,6 +16,9 @@ from sqlglot.helper import seq_get
 from sqlglot.time import format_time
 from sqlglot.tokens import TokenType
 
+if t.TYPE_CHECKING:
+    from sqlglot._typing import E
+
 FULL_FORMAT_TIME_MAPPING = {
     "weekday": "%A",
     "dw": "%A",
@@ -50,16 +53,20 @@ DATE_FMT_RE = re.compile("([dD]{1,2})|([mM]{1,2})|([yY]{1,4})|([hH]{1,2})|([sS]{
 TRANSPILE_SAFE_NUMBER_FMT = {"N", "C"}
 
 
-def _format_time_lambda(exp_class, full_format_mapping=None, default=None):
-    def _format_time(args):
+def _format_time_lambda(
+    exp_class: t.Type[E], full_format_mapping: t.Optional[bool] = None
+) -> t.Callable[[t.List], E]:
+    def _format_time(args: t.List) -> E:
+        assert len(args) == 2
+
         return exp_class(
-            this=seq_get(args, 1),
+            this=args[1],
             format=exp.Literal.string(
                 format_time(
-                    seq_get(args, 0).name or (TSQL.time_format if default is True else default),
-                    {**TSQL.time_mapping, **FULL_FORMAT_TIME_MAPPING}
+                    args[0].name,
+                    {**TSQL.TIME_MAPPING, **FULL_FORMAT_TIME_MAPPING}
                     if full_format_mapping
-                    else TSQL.time_mapping,
+                    else TSQL.TIME_MAPPING,
                 )
             ),
         )
@@ -67,22 +74,26 @@ def _format_time_lambda(exp_class, full_format_mapping=None, default=None):
     return _format_time
 
 
-def _parse_format(args):
-    fmt = seq_get(args, 1)
-    number_fmt = fmt.name in TRANSPILE_SAFE_NUMBER_FMT or not DATE_FMT_RE.search(fmt.this)
+def _parse_format(args: t.List) -> exp.Expression:
+    assert len(args) == 2
+
+    fmt = args[1]
+    number_fmt = fmt.name in TRANSPILE_SAFE_NUMBER_FMT or not DATE_FMT_RE.search(fmt.name)
+
     if number_fmt:
-        return exp.NumberToStr(this=seq_get(args, 0), format=fmt)
+        return exp.NumberToStr(this=args[0], format=fmt)
+
     return exp.TimeToStr(
-        this=seq_get(args, 0),
+        this=args[0],
         format=exp.Literal.string(
-            format_time(fmt.name, TSQL.format_time_mapping)
+            format_time(fmt.name, TSQL.FORMAT_TIME_MAPPING)
             if len(fmt.name) == 1
-            else format_time(fmt.name, TSQL.time_mapping)
+            else format_time(fmt.name, TSQL.TIME_MAPPING)
         ),
     )
 
 
-def _parse_eomonth(args):
+def _parse_eomonth(args: t.List) -> exp.Expression:
     date = seq_get(args, 0)
     month_lag = seq_get(args, 1)
     unit = DATE_DELTA_INTERVAL.get("month")
@@ -96,7 +107,7 @@ def _parse_eomonth(args):
     return exp.LastDateOfMonth(this=exp.DateAdd(this=date, expression=month_lag, unit=unit))
 
 
-def _parse_hashbytes(args):
+def _parse_hashbytes(args: t.List) -> exp.Expression:
     kind, data = args
     kind = kind.name.upper() if kind.is_string else ""
 
@@ -110,48 +121,55 @@ def _parse_hashbytes(args):
         return exp.SHA2(this=data, length=exp.Literal.number(256))
     if kind == "SHA2_512":
         return exp.SHA2(this=data, length=exp.Literal.number(512))
+
     return exp.func("HASHBYTES", *args)
 
 
-def generate_date_delta_with_unit_sql(self, e):
-    func = "DATEADD" if isinstance(e, exp.DateAdd) else "DATEDIFF"
-    return self.func(func, e.text("unit"), e.expression, e.this)
+def generate_date_delta_with_unit_sql(
+    self: generator.Generator, expression: exp.DateAdd | exp.DateDiff
+) -> str:
+    func = "DATEADD" if isinstance(expression, exp.DateAdd) else "DATEDIFF"
+    return self.func(func, expression.text("unit"), expression.expression, expression.this)
 
 
-def _format_sql(self, e):
+def _format_sql(self: generator.Generator, expression: exp.NumberToStr | exp.TimeToStr) -> str:
     fmt = (
-        e.args["format"]
-        if isinstance(e, exp.NumberToStr)
-        else exp.Literal.string(format_time(e.text("format"), TSQL.inverse_time_mapping))
+        expression.args["format"]
+        if isinstance(expression, exp.NumberToStr)
+        else exp.Literal.string(
+            format_time(
+                expression.text("format"), t.cast(t.Dict[str, str], TSQL.INVERSE_TIME_MAPPING)
+            )
+        )
     )
-    return self.func("FORMAT", e.this, fmt)
+    return self.func("FORMAT", expression.this, fmt)
 
 
-def _string_agg_sql(self, e):
-    e = e.copy()
+def _string_agg_sql(self: generator.Generator, expression: exp.GroupConcat) -> str:
+    expression = expression.copy()
 
-    this = e.this
-    distinct = e.find(exp.Distinct)
+    this = expression.this
+    distinct = expression.find(exp.Distinct)
     if distinct:
         # exp.Distinct can appear below an exp.Order or an exp.GroupConcat expression
         self.unsupported("T-SQL STRING_AGG doesn't support DISTINCT.")
         this = distinct.pop().expressions[0]
 
     order = ""
-    if isinstance(e.this, exp.Order):
-        if e.this.this:
-            this = e.this.this.pop()
-        order = f" WITHIN GROUP ({self.sql(e.this)[1:]})"  # Order has a leading space
+    if isinstance(expression.this, exp.Order):
+        if expression.this.this:
+            this = expression.this.this.pop()
+        order = f" WITHIN GROUP ({self.sql(expression.this)[1:]})"  # Order has a leading space
 
-    separator = e.args.get("separator") or exp.Literal.string(",")
+    separator = expression.args.get("separator") or exp.Literal.string(",")
     return f"STRING_AGG({self.format_args(this, separator)}){order}"
 
 
 class TSQL(Dialect):
-    null_ordering = "nulls_are_small"
-    time_format = "'yyyy-mm-dd hh:mm:ss'"
+    NULL_ORDERING = "nulls_are_small"
+    TIME_FORMAT = "'yyyy-mm-dd hh:mm:ss'"
 
-    time_mapping = {
+    TIME_MAPPING = {
         "year": "%Y",
         "qq": "%q",
         "q": "%q",
@@ -195,7 +213,7 @@ class TSQL(Dialect):
         "yy": "%y",
     }
 
-    convert_format_mapping = {
+    CONVERT_FORMAT_MAPPING = {
         "0": "%b %d %Y %-I:%M%p",
         "1": "%m/%d/%y",
         "2": "%y.%m.%d",
@@ -235,8 +253,8 @@ class TSQL(Dialect):
         "120": "%Y-%m-%d %H:%M:%S",
         "121": "%Y-%m-%d %H:%M:%S.%f",
     }
-    # not sure if complete
-    format_time_mapping = {
+
+    FORMAT_TIME_MAPPING = {
         "y": "%B %Y",
         "d": "%m/%d/%Y",
         "H": "%-H",
@@ -292,11 +310,9 @@ class TSQL(Dialect):
 
     class Parser(parser.Parser):
         FUNCTIONS = {
-            **parser.Parser.FUNCTIONS,  # type: ignore
+            **parser.Parser.FUNCTIONS,
             "CHARINDEX": lambda args: exp.StrPosition(
-                this=seq_get(args, 1),
-                substr=seq_get(args, 0),
-                position=seq_get(args, 2),
+                this=seq_get(args, 1), substr=seq_get(args, 0), position=seq_get(args, 2)
             ),
             "DATEADD": parse_date_delta(exp.DateAdd, unit_mapping=DATE_DELTA_INTERVAL),
             "DATEDIFF": parse_date_delta(exp.DateDiff, unit_mapping=DATE_DELTA_INTERVAL),
@@ -332,19 +348,21 @@ class TSQL(Dialect):
             DataType.Type.NCHAR,
         }
 
-        RETURNS_TABLE_TOKENS = parser.Parser.ID_VAR_TOKENS - {  # type: ignore
+        RETURNS_TABLE_TOKENS = parser.Parser.ID_VAR_TOKENS - {
             TokenType.TABLE,
-            *parser.Parser.TYPE_TOKENS,  # type: ignore
+            *parser.Parser.TYPE_TOKENS,
         }
 
         STATEMENT_PARSERS = {
-            **parser.Parser.STATEMENT_PARSERS,  # type: ignore
+            **parser.Parser.STATEMENT_PARSERS,
             TokenType.END: lambda self: self._parse_command(),
         }
 
         LOG_BASE_FIRST = False
         LOG_DEFAULTS_TO_LN = True
 
+        CONCAT_NULL_OUTPUTS_STRING = True
+
         def _parse_system_time(self) -> t.Optional[exp.Expression]:
             if not self._match_text_seq("FOR", "SYSTEM_TIME"):
                 return None
@@ -377,12 +395,12 @@ class TSQL(Dialect):
 
             return system_time
 
-        def _parse_table_parts(self, schema: bool = False) -> exp.Expression:
+        def _parse_table_parts(self, schema: bool = False) -> exp.Table:
             table = super()._parse_table_parts(schema=schema)
             table.set("system_time", self._parse_system_time())
             return table
 
-        def _parse_returns(self) -> exp.Expression:
+        def _parse_returns(self) -> exp.ReturnsProperty:
             table = self._parse_id_var(any_token=False, tokens=self.RETURNS_TABLE_TOKENS)
             returns = super()._parse_returns()
             returns.set("table", table)
@@ -405,12 +423,12 @@ class TSQL(Dialect):
                 format_val = self._parse_number()
                 format_val_name = format_val.name if format_val else ""
 
-                if format_val_name not in TSQL.convert_format_mapping:
+                if format_val_name not in TSQL.CONVERT_FORMAT_MAPPING:
                     raise ValueError(
                         f"CONVERT function at T-SQL does not support format style {format_val_name}"
                     )
 
-                format_norm = exp.Literal.string(TSQL.convert_format_mapping[format_val_name])
+                format_norm = exp.Literal.string(TSQL.CONVERT_FORMAT_MAPPING[format_val_name])
 
                 # Check whether the convert entails a string to date format
                 if to.this == DataType.Type.DATE:
@@ -450,7 +468,7 @@ class TSQL(Dialect):
         LOCKING_READS_SUPPORTED = True
 
         TYPE_MAPPING = {
-            **generator.Generator.TYPE_MAPPING,  # type: ignore
+            **generator.Generator.TYPE_MAPPING,
             exp.DataType.Type.INT: "INTEGER",
             exp.DataType.Type.DECIMAL: "NUMERIC",
             exp.DataType.Type.DATETIME: "DATETIME2",
@@ -458,7 +476,7 @@ class TSQL(Dialect):
         }
 
         TRANSFORMS = {
-            **generator.Generator.TRANSFORMS,  # type: ignore
+            **generator.Generator.TRANSFORMS,
             exp.DateAdd: generate_date_delta_with_unit_sql,
             exp.DateDiff: generate_date_delta_with_unit_sql,
             exp.CurrentDate: rename_func("GETDATE"),
@@ -480,7 +498,7 @@ class TSQL(Dialect):
         TRANSFORMS.pop(exp.ReturnsProperty)
 
         PROPERTIES_LOCATION = {
-            **generator.Generator.PROPERTIES_LOCATION,  # type: ignore
+            **generator.Generator.PROPERTIES_LOCATION,
             exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
         }
 
