@@ -360,11 +360,19 @@ def _ordered_py(self, expression):
 
 def _rename(self, e):
     try:
-        if "expressions" in e.args:
-            this = self.sql(e, "this")
-            this = f"{this}, " if this else ""
-            return f"{e.key.upper()}({this}{self.expressions(e)})"
-        return self.func(e.key, *e.args.values())
+        values = list(e.args.values())
+
+        if len(values) == 1:
+            values = values[0]
+            if not isinstance(values, list):
+                return self.func(e.key, values)
+            return self.func(e.key, *values)
+
+        if isinstance(e, exp.Func) and e.is_var_len_args:
+            *head, tail = values
+            return self.func(e.key, *head, *tail)
+
+        return self.func(e.key, *values)
     except Exception as ex:
         raise Exception(f"Could not rename {repr(e)}") from ex
 
@@ -386,7 +394,7 @@ def _lambda_sql(self, e: exp.Lambda) -> str:
     names = {e.name.lower() for e in e.expressions}
 
     e = e.transform(
-        lambda n: exp.Var(this=n.name)
+        lambda n: exp.var(n.name)
         if isinstance(n, exp.Identifier) and n.name.lower() in names
         else n
     )
@@ -412,7 +420,7 @@ class Python(Dialect):
             exp.Column: lambda self, e: f"scope[{self.sql(e, 'table') or None}][{self.sql(e.this)}]",
             exp.Distinct: lambda self, e: f"set({self.sql(e, 'this')})",
             exp.Extract: lambda self, e: f"EXTRACT('{e.name.lower()}', {self.sql(e, 'expression')})",
-            exp.In: lambda self, e: f"{self.sql(e, 'this')} in ({self.expressions(e, flat=True)})",
+            exp.In: lambda self, e: f"{self.sql(e, 'this')} in {{{self.expressions(e, flat=True)}}}",
             exp.Interval: lambda self, e: f"INTERVAL({self.sql(e.this)}, '{self.sql(e.unit)}')",
             exp.Is: lambda self, e: self.binary(e, "is"),
             exp.Lambda: _lambda_sql,
